from setuptools import setup, find_packages

# Configuration
NAME = "io4it"
VERSION = "3.0.5.993"

INSTALL_REQUIRES = [
    "torchvision",
    "torch",
    "pylatexenc",
    "docopt",
    "boto3",
    "opencv-python-headless",
    "docling",
    "docling-core", 
    "whisper",
    "whisper-openai",
    "pypandoc",
    "pypandoc-binary",
    "scikit-learn",
    "openai",
    "pip-system-certs",
    "docx2pdf",
    "msal",
    "exchangelib",
    "html2text",
    "ddgs",
    "CATEGORIT"
]

AUTHOR = ""
AUTHOR_EMAIL = ""
URL = ""
DESCRIPTION = ""
LICENSE = ""
KEYWORDS = ["orange3 add-on",]

PACKAGES = find_packages()
PACKAGES = [pack for pack in PACKAGES if "orangecontrib" in pack and "IO4IT" in pack]
PACKAGES.append("orangecontrib")

PACKAGE_DATA = {
    "orangecontrib.IO4IT.widgets": ["icons/*", "designer/*","../utils/config.json","../toolViews/*","../toolViews/designer/*"],
}


ENTRY_POINTS = {
    "orange.widgets": (
        "IO4IT = orangecontrib.IO4IT.widgets",
        "AAIT - ALGORITHM = orangecontrib.ALGORITHM.widgets",
        "AAIT - API = orangecontrib.API.widgets",
        "AAIT - MODELS = orangecontrib.LLM_MODELS.widgets",
        "AAIT - LLM INTEGRATION = orangecontrib.LLM_INTEGRATION.widgets",
        "AAIT - TOOLBOX = orangecontrib.TOOLBOX.widgets",
    ),
}

NAMESPACE_PACKAGES = ["orangecontrib"]


setup(
    name=NAME,
    version=VERSION,
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    url=URL,
    license=LICENSE,
    keywords=KEYWORDS,
    packages=PACKAGES,
    package_data=PACKAGE_DATA,
    install_requires=INSTALL_REQUIRES,
    entry_points=ENTRY_POINTS,
    namespace_packages=NAMESPACE_PACKAGES,
)
